searchData={"items":[{"type":"module","title":"Qu","doc":"`Qu` is an Elixir library that provides a single interface to four different types of\nqueues: FIFO, LIFO, circular, and priority.","ref":"Qu.html"},{"type":"module","title":"Usage - Qu","doc":"All of the queue types support the same operations: `Qu.put/2` to add an item, \n`Qu.pop/1` to remove the head, `Qu.peek/1` to view the head without removal, \n`Qu.pull/2` to remove multiple items, and `Qu.size/1` to get the number of \nitems in the queue. These operations are demonstrated below. For more details\nabout the queue operations, see the documentation for each function.\n\nTo create and fill a FIFO queue with a maximum size of 3, use `Qu.new/2` and\nthen apply `Qu.put/2` in a reduction:\n\n```elixir\nfifo = Qu.new(:fifo, max_size: 3) \n#=> #FIFO \n\nfifo =\n  Enum.reduce([\"a\", \"b\", \"c\"], fifo, fn item, q ->\n    {:ok, q} = Qu.put(q, item)\n    q\n  end)\n#=> #FIFO \n```\n\nA priority queue expects items of the form `{priority_key, value}`. Here is\nhow to create and populate a priority queue where items are popped in \nascending order of priority:\n\n```elixir\nprio = Qu.new(:priority, priority_order: :asc)\n\nEnum.reduce([{1, \"a\"}, {2, \"b\"}, {3, \"c\"}], prio, fn item, q ->\n  {:ok, q} = Qu.put(q, item)\n  q\nend)\n#=> #Priority , size: 3, max_size: nil>\n```\n\nFor FIFO, LIFO, and priority queues, applying `Qu.put/2` to a full queue returns\n`:error`:\n\n```elixir\nQu.new(:fifo, max_size: 0) |> Qu.put(fifo, \"a\")\n#=> :error\n```\n\nAn error is never returned when adding items to a circular queue, since the \noldest item is discarded when inserting into a full circular queue. To preserve\nthe common interface, `Qu.put/2` still returns `{:ok, updated_queue}` for a\ncircular queue.\n\nThe head of the queue can be removed and returned with `Qu.pop/1`:\n\n```elixir\n{:ok, fifo} = Qu.new(:fifo) |> Qu.put(\"a\")\nQu.pop(fifo)\n#=> {:ok, \"a\", #FIFO }\n```\n\nFor all queue types, `Qu.pop/1` returns `:error` if the queue is empty:\n\n```elixir\nQu.new(:fifo, max_size: 0) |> Qu.pop()\n#=> :error\n```\n\nThe head can be viewed without modifying the queue using `Qu.peek/1`:\n\n```elixir\n{:ok, fifo} = Qu.new(:fifo) |> Qu.put(\"a\")\nQu.peek(fifo)\n#=> {:ok, \"a\"}\n```\n\nSimilar to `Qu.pop/1`, `Qu.peek/1` will return `:error` if the queue is empty.\n\nUse `Qu.pull/2` to remove and return multiple items:\n\n```elixir\nfifo = Qu.new(:fifo, max_size: 3)\n\n[\"a\", \"b\", \"c\"]\n|> Enum.reduce(fifo, fn item, q ->\n  {:ok, q} = Qu.put(q, item)\n  q\nend)\n|> Qu.pull(3)\n#=> {[\"a\", \"b\", \"c\"], #FIFO }\n```\n\nNote that `Qu.pull/2` does not return `:error` if the queue is empty or if the\nnumber of elements pulled is larger than the queue size.\n\nFor all of the queue types, `Qu.size/1` returns the current size of the queue:\n\n```elixir\n{:ok, fifo} = Qu.new(:fifo) |> Qu.put(\"a\")\nQu.size(fifo)\n#=> 1\n```\n\nAll of the queue types also implement the `Enumerable` and `Collectable` \nprotocols, so that `Enum.reduce/3`, `Enum.to_list/1`, and `Enum.into/2` may\nbe used.","ref":"Qu.html#module-usage"},{"type":"module","title":"Supported queue types - Qu","doc":"Listed below are the supported queue types, as well as notes about their\nimplementations and usage guidelines.\n\n#","ref":"Qu.html#module-supported-queue-types"},{"type":"module","title":"FIFO - Qu","doc":"A first-in, first-out queue, in which the oldest item in the queue will \nbe the next item removed. If the queue has a finite maximum size and is full,\nthe next insert will fail.\n\nThe implementation follows the \n[Erlang queue](https://www.erlang.org/doc/man/queue) and uses separate read and\nwrite lists to achieve `O(1)` amortized time for insert and delete operations.\n\n#","ref":"Qu.html#module-fifo"},{"type":"module","title":"LIFO - Qu","doc":"A last-in, first-out queue, in which the item most recently added to the \nqueue will be the next item removed. This is also known as a _stack_. If the \nqueue has a finite maximum size and is full, the next insert will fail.\n\nThe implementation uses a single Elixir list and has `O(1)` time for both \ninsert and delete operations.\n\n#","ref":"Qu.html#module-lifo"},{"type":"module","title":"Circular - Qu","doc":"A circular queue is similar to a FIFO queue in that the oldest item in the \nqueue will be the next item removed. However, when adding an item to a full \ncircular queue, the oldest item will be discarded to preserve the fixed size. \nAn insert in a full circular queue will always succeed.\n\nThe implementation borrows from the FIFO implementation, but has a modified\n`pop/1` function to achieve the properties described above.\n\n#","ref":"Qu.html#module-circular"},{"type":"module","title":"Priority - Qu","doc":"In a priority queue, items are added as key-value pairs, where the key is a\nsortable priority. Items are removed in priority order, either descending or \nascending, depending on the configuration.\n\nThe implementation uses a heap, specifically a pairing heap \n(see the [`PairingHeap`](https://hexdocs.pm/pairing_heap/PairingHeap.html) \nElixir library), which has `O(1)` insert time and `O(log n)` amortized deletion\n time.","ref":"Qu.html#module-priority"},{"type":"function","title":"Qu.new/2","doc":"Create a new queue of the given type and maximum size.","ref":"Qu.html#new/2"},{"type":"function","title":"Options - Qu.new/2","doc":"* `:max_size` - Maximum size of the queue. A value of `nil` indicates that the queue has\n  no maximum size. When the queue type is `:circular`, the maximum must be\n  a positive integer. The default value is `nil`.\n\n* `:priority_order` - Key order in which items are removed when the queue type is\n  `:priority`. When a tuple like `{:desc, DateTime}` is given, the key is\n  assumed to be a `DateTime`, and `DateTime.compare/2` is used compare\n  keys. The default value is `:asc`.","ref":"Qu.html#new/2-options"},{"type":"function","title":"Examples - Qu.new/2","doc":"iex> Qu.new(:fifo, max_size: 10)\n    #FIFO \n\n    iex> Qu.new(:priority, priority_order: {:asc, DateTime})\n    #Priority , size: 0, max_size: nil>","ref":"Qu.html#new/2-examples"},{"type":"function","title":"Qu.peek/1","doc":"Get the item at the head of the queue.\n\nIf there is at least one item in the queue, this returns\n`{:ok, item}`. If the queue is empty, `:error` is returned.","ref":"Qu.html#peek/1"},{"type":"function","title":"Examples - Qu.peek/1","doc":"iex> {:ok, q} = Qu.new(:fifo, max_size: 1) |> Qu.put(\"a\")\n    iex> {:ok, \"a\"} = Qu.peek(q)","ref":"Qu.html#peek/1-examples"},{"type":"function","title":"Qu.pop/1","doc":"Pop the item at the head of the queue.\n\nIf there is at least one item in the queue, this returns\n`{:ok, item, updated_queue}`. If the queue is empty, `:error` is returned.","ref":"Qu.html#pop/1"},{"type":"function","title":"Examples - Qu.pop/1","doc":"iex> {:ok, q} = Qu.new(:fifo) |> Qu.put(\"a\")\n    iex> {:ok, \"a\", q} = Qu.pop(q)\n    iex> q\n    #FIFO \n    iex> Qu.pop(q)\n    :error","ref":"Qu.html#pop/1-examples"},{"type":"function","title":"Qu.pull/2","doc":"Return the first `n` items in the queue and the final state of the heap.\n\nIf the queue size is less than `n`, all items are returned along with\nan empty heap.\n\nNote that `pull` is often more convenient than `pop`, because of it can pop\nmultiple items at once, and because it never returns `:error`.","ref":"Qu.html#pull/2"},{"type":"function","title":"Examples - Qu.pull/2","doc":"iex> {:ok, q} = Qu.new(:fifo, max_size: 1) |> Qu.put(\"a\")\n    iex> {[\"a\"], _} = Qu.pull(q, 1)","ref":"Qu.html#pull/2-examples"},{"type":"function","title":"Qu.put/2","doc":"Insert an item into the queue.\n\nIf the queue size is less than the maximum size, this returns\n`{:ok, updated_queue}`. With the exception of a cirular queue, if the queue\nsize equals the maximum size, `:error` is returned. For a circular queue,\n`Qu.put/2` never returns `:error`.","ref":"Qu.html#put/2"},{"type":"function","title":"Examples - Qu.put/2","doc":"iex> {:ok, q} = Qu.new(:fifo, max_size: 1) |> Qu.put(\"a\")\n    iex> q\n    #FIFO \n    iex> Qu.put(q, \"b\")\n    :error\n\n    iex> {:ok, q}  = Qu.new(:priority, priority_order: :asc) |> Qu.put({1, \"a\"})\n    iex> q\n    #Priority , size: 1, max_size: nil>","ref":"Qu.html#put/2-examples"},{"type":"function","title":"Qu.size/1","doc":"Return the size of the queue.","ref":"Qu.html#size/1"},{"type":"function","title":"Examples - Qu.size/1","doc":"iex> q = Qu.new(:fifo, max_size: 1)\n    iex> Qu.size(q)\n    0\n    iex> {:ok, q} = Qu.put(q, \"a\")\n    iex> Qu.size(q)\n    1","ref":"Qu.html#size/1-examples"},{"type":"type","title":"Qu.item/0","doc":"A queue item, either a single value or, in the case of a priority queue, a\ntuple of a priority key and a value.","ref":"Qu.html#t:item/0"},{"type":"type","title":"Qu.max_size/0","doc":"Maximum queue size. A value of `nil` indicates that the queue has no maximum\nsize.","ref":"Qu.html#t:max_size/0"},{"type":"type","title":"Qu.queue_type/0","doc":"Type of queue.","ref":"Qu.html#t:queue_type/0"},{"type":"type","title":"Qu.size/0","doc":"Current size of the queue.","ref":"Qu.html#t:size/0"},{"type":"module","title":"Qu.Circular","doc":"Circular queue implementation.\n\nA circular queue is implemented by delegating to FIFO, but with a change to\nthe logic of how items are inserted. When inserting into a circular queue, if\nthe circular queue is full, the head is first popped before new item is\ninserted.","ref":"Qu.Circular.html"},{"type":"function","title":"Qu.Circular.new/1","doc":"","ref":"Qu.Circular.html#new/1"},{"type":"function","title":"Qu.Circular.peek/1","doc":"","ref":"Qu.Circular.html#peek/1"},{"type":"function","title":"Qu.Circular.pop/1","doc":"","ref":"Qu.Circular.html#pop/1"},{"type":"function","title":"Qu.Circular.put/2","doc":"","ref":"Qu.Circular.html#put/2"},{"type":"function","title":"Qu.Circular.size/1","doc":"","ref":"Qu.Circular.html#size/1"},{"type":"type","title":"Qu.Circular.item/0","doc":"","ref":"Qu.Circular.html#t:item/0"},{"type":"type","title":"Qu.Circular.max_size/0","doc":"","ref":"Qu.Circular.html#t:max_size/0"},{"type":"type","title":"Qu.Circular.size/0","doc":"","ref":"Qu.Circular.html#t:size/0"},{"type":"type","title":"Qu.Circular.t/0","doc":"","ref":"Qu.Circular.html#t:t/0"},{"type":"module","title":"Qu.FIFO","doc":"FIFO queue implementation.","ref":"Qu.FIFO.html"},{"type":"function","title":"Qu.FIFO.new/1","doc":"","ref":"Qu.FIFO.html#new/1"},{"type":"function","title":"Qu.FIFO.peek/1","doc":"","ref":"Qu.FIFO.html#peek/1"},{"type":"function","title":"Qu.FIFO.pop/1","doc":"","ref":"Qu.FIFO.html#pop/1"},{"type":"function","title":"Qu.FIFO.put/2","doc":"","ref":"Qu.FIFO.html#put/2"},{"type":"function","title":"Qu.FIFO.size/1","doc":"","ref":"Qu.FIFO.html#size/1"},{"type":"type","title":"Qu.FIFO.item/0","doc":"","ref":"Qu.FIFO.html#t:item/0"},{"type":"type","title":"Qu.FIFO.max_size/0","doc":"","ref":"Qu.FIFO.html#t:max_size/0"},{"type":"type","title":"Qu.FIFO.size/0","doc":"","ref":"Qu.FIFO.html#t:size/0"},{"type":"type","title":"Qu.FIFO.t/0","doc":"","ref":"Qu.FIFO.html#t:t/0"},{"type":"module","title":"Qu.LIFO","doc":"LIFO queue implementation.","ref":"Qu.LIFO.html"},{"type":"function","title":"Qu.LIFO.new/1","doc":"","ref":"Qu.LIFO.html#new/1"},{"type":"function","title":"Qu.LIFO.peek/1","doc":"","ref":"Qu.LIFO.html#peek/1"},{"type":"function","title":"Qu.LIFO.pop/1","doc":"","ref":"Qu.LIFO.html#pop/1"},{"type":"function","title":"Qu.LIFO.put/2","doc":"","ref":"Qu.LIFO.html#put/2"},{"type":"function","title":"Qu.LIFO.size/1","doc":"","ref":"Qu.LIFO.html#size/1"},{"type":"type","title":"Qu.LIFO.item/0","doc":"","ref":"Qu.LIFO.html#t:item/0"},{"type":"type","title":"Qu.LIFO.max_size/0","doc":"","ref":"Qu.LIFO.html#t:max_size/0"},{"type":"type","title":"Qu.LIFO.size/0","doc":"","ref":"Qu.LIFO.html#t:size/0"},{"type":"type","title":"Qu.LIFO.t/0","doc":"","ref":"Qu.LIFO.html#t:t/0"},{"type":"module","title":"Qu.Priority","doc":"Priority queue implementation.","ref":"Qu.Priority.html"},{"type":"function","title":"Qu.Priority.new/2","doc":"","ref":"Qu.Priority.html#new/2"},{"type":"function","title":"Qu.Priority.peek/1","doc":"","ref":"Qu.Priority.html#peek/1"},{"type":"function","title":"Qu.Priority.pop/1","doc":"","ref":"Qu.Priority.html#pop/1"},{"type":"function","title":"Qu.Priority.put/2","doc":"","ref":"Qu.Priority.html#put/2"},{"type":"function","title":"Qu.Priority.size/1","doc":"","ref":"Qu.Priority.html#size/1"},{"type":"type","title":"Qu.Priority.item/0","doc":"","ref":"Qu.Priority.html#t:item/0"},{"type":"type","title":"Qu.Priority.key/0","doc":"","ref":"Qu.Priority.html#t:key/0"},{"type":"type","title":"Qu.Priority.max_size/0","doc":"","ref":"Qu.Priority.html#t:max_size/0"},{"type":"type","title":"Qu.Priority.order/0","doc":"","ref":"Qu.Priority.html#t:order/0"},{"type":"type","title":"Qu.Priority.size/0","doc":"","ref":"Qu.Priority.html#t:size/0"},{"type":"type","title":"Qu.Priority.t/0","doc":"","ref":"Qu.Priority.html#t:t/0"},{"type":"type","title":"Qu.Priority.value/0","doc":"","ref":"Qu.Priority.html#t:value/0"},{"type":"protocol","title":"Qu.Queue","doc":"Protocol definition shared by all queue implementations.","ref":"Qu.Queue.html"},{"type":"function","title":"Qu.Queue.peek/1","doc":"","ref":"Qu.Queue.html#peek/1"},{"type":"function","title":"Qu.Queue.pop/1","doc":"","ref":"Qu.Queue.html#pop/1"},{"type":"function","title":"Qu.Queue.put/2","doc":"","ref":"Qu.Queue.html#put/2"},{"type":"function","title":"Qu.Queue.size/1","doc":"","ref":"Qu.Queue.html#size/1"},{"type":"type","title":"Qu.Queue.item/0","doc":"","ref":"Qu.Queue.html#t:item/0"},{"type":"type","title":"Qu.Queue.t/0","doc":"All the types that implement this protocol.","ref":"Qu.Queue.html#t:t/0"},{"type":"extras","title":"README","doc":"[![Hex.pm Version](https://img.shields.io/hexpm/v/qu.svg)](https://hex.pm/packages/qu)\n\n<!-- END HEADER -->\n\n`Qu` is an Elixir library that provides a single interface to four different types of\nqueues: FIFO, LIFO, circular, and priority.","ref":"readme.html"},{"type":"extras","title":"Usage - README","doc":"All of the queue types support the same operations: `Qu.put/2` to add an item, \n`Qu.pop/1` to remove the head, `Qu.peek/1` to view the head without removal, \n`Qu.pull/2` to remove multiple items, and `Qu.size/1` to get the number of \nitems in the queue. These operations are demonstrated below. For more details\nabout the queue operations, see the documentation for each function.\n\nTo create and fill a FIFO queue with a maximum size of 3, use `Qu.new/2` and\nthen apply `Qu.put/2` in a reduction:\n\n```elixir\nfifo = Qu.new(:fifo, max_size: 3) \n#=> #FIFO \n\nfifo =\n  Enum.reduce([\"a\", \"b\", \"c\"], fifo, fn item, q ->\n    {:ok, q} = Qu.put(q, item)\n    q\n  end)\n#=> #FIFO \n```\n\nA priority queue expects items of the form `{priority_key, value}`. Here is\nhow to create and populate a priority queue where items are popped in \nascending order of priority:\n\n```elixir\nprio = Qu.new(:priority, priority_order: :asc)\n\nEnum.reduce([{1, \"a\"}, {2, \"b\"}, {3, \"c\"}], prio, fn item, q ->\n  {:ok, q} = Qu.put(q, item)\n  q\nend)\n#=> #Priority , size: 3, max_size: nil>\n```\n\nFor FIFO, LIFO, and priority queues, applying `Qu.put/2` to a full queue returns\n`:error`:\n\n```elixir\nQu.new(:fifo, max_size: 0) |> Qu.put(fifo, \"a\")\n#=> :error\n```\n\nAn error is never returned when adding items to a circular queue, since the \noldest item is discarded when inserting into a full circular queue. To preserve\nthe common interface, `Qu.put/2` still returns `{:ok, updated_queue}` for a\ncircular queue.\n\nThe head of the queue can be removed and returned with `Qu.pop/1`:\n\n```elixir\n{:ok, fifo} = Qu.new(:fifo) |> Qu.put(\"a\")\nQu.pop(fifo)\n#=> {:ok, \"a\", #FIFO }\n```\n\nFor all queue types, `Qu.pop/1` returns `:error` if the queue is empty:\n\n```elixir\nQu.new(:fifo, max_size: 0) |> Qu.pop()\n#=> :error\n```\n\nThe head can be viewed without modifying the queue using `Qu.peek/1`:\n\n```elixir\n{:ok, fifo} = Qu.new(:fifo) |> Qu.put(\"a\")\nQu.peek(fifo)\n#=> {:ok, \"a\"}\n```\n\nSimilar to `Qu.pop/1`, `Qu.peek/1` will return `:error` if the queue is empty.\n\nUse `Qu.pull/2` to remove and return multiple items:\n\n```elixir\nfifo = Qu.new(:fifo, max_size: 3)\n\n[\"a\", \"b\", \"c\"]\n|> Enum.reduce(fifo, fn item, q ->\n  {:ok, q} = Qu.put(q, item)\n  q\nend)\n|> Qu.pull(3)\n#=> {[\"a\", \"b\", \"c\"], #FIFO }\n```\n\nNote that `Qu.pull/2` does not return `:error` if the queue is empty or if the\nnumber of elements pulled is larger than the queue size.\n\nFor all of the queue types, `Qu.size/1` returns the current size of the queue:\n\n```elixir\n{:ok, fifo} = Qu.new(:fifo) |> Qu.put(\"a\")\nQu.size(fifo)\n#=> 1\n```\n\nAll of the queue types also implement the `Enumerable` and `Collectable` \nprotocols, so that `Enum.reduce/3`, `Enum.to_list/1`, and `Enum.into/2` may\nbe used.","ref":"readme.html#usage"},{"type":"extras","title":"Supported queue types - README","doc":"Listed below are the supported queue types, as well as notes about their\nimplementations and usage guidelines.\n\n#","ref":"readme.html#supported-queue-types"},{"type":"extras","title":"FIFO - README","doc":"A first-in, first-out queue, in which the oldest item in the queue will \nbe the next item removed. If the queue has a finite maximum size and is full,\nthe next insert will fail.\n\nThe implementation follows the \n[Erlang queue](https://www.erlang.org/doc/man/queue) and uses separate read and\nwrite lists to achieve `O(1)` amortized time for insert and delete operations.\n\n#","ref":"readme.html#fifo"},{"type":"extras","title":"LIFO - README","doc":"A last-in, first-out queue, in which the item most recently added to the \nqueue will be the next item removed. This is also known as a _stack_. If the \nqueue has a finite maximum size and is full, the next insert will fail.\n\nThe implementation uses a single Elixir list and has `O(1)` time for both \ninsert and delete operations.\n\n#","ref":"readme.html#lifo"},{"type":"extras","title":"Circular - README","doc":"A circular queue is similar to a FIFO queue in that the oldest item in the \nqueue will be the next item removed. However, when adding an item to a full \ncircular queue, the oldest item will be discarded to preserve the fixed size. \nAn insert in a full circular queue will always succeed.\n\nThe implementation borrows from the FIFO implementation, but has a modified\n`pop/1` function to achieve the properties described above.\n\n#","ref":"readme.html#circular"},{"type":"extras","title":"Priority - README","doc":"In a priority queue, items are added as key-value pairs, where the key is a\nsortable priority. Items are removed in priority order, either descending or \nascending, depending on the configuration.\n\nThe implementation uses a heap, specifically a pairing heap \n(see the [`PairingHeap`](https://hexdocs.pm/pairing_heap/PairingHeap.html) \nElixir library), which has `O(1)` insert time and `O(log n)` amortized deletion\n time.","ref":"readme.html#priority"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}